@@ -4,7 +4,7 @@ Custom data objects
 from __future__ import unicode_literals
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models.loading import get_model
-from django.utils.encoding import StrAndUnicode
+from django.utils.encoding import python_2_unicode_compatible
 import logging
 
 try:
@@ -19,7 +19,8 @@ except ImportError:
 logger = logging.getLogger('any_urlfield.models')
 
 
-class AnyUrlValue(StrAndUnicode):
+@python_2_unicode_compatible
+class AnyUrlValue(object):
     """
     Custom value object for the :class:`~any_urlfield.models.AnyUrlField`.
     This value holds both the internal page ID, and external URL.
@@ -150,8 +151,8 @@ class AnyUrlValue(StrAndUnicode):
         return self.url_type.prefix
 
 
-    # Python 3 support comes from StrAndUnicode
-    def __unicode__(self):
+    # Python 2 support comes from python_2_unicode_compatible
+    def __str__(self):
         """
         Return the URL that the value points to.
         """
@@ -165,7 +166,7 @@ class AnyUrlValue(StrAndUnicode):
                 return object.get_absolute_url()
             except ObjectDoesNotExist as e:
                 # Silently fail in templates. Avoid full page crashing.
-                logger.exception("Failed to generate URL for {0}".format(repr(self)))
+                logger.error("Failed to generate URL for %r: %s", self, e)
                 return "#{0}".format(e.__class__.__name__)
         else:
             return self.type_value or ""
@@ -193,3 +194,40 @@ class AnyUrlValue(StrAndUnicode):
 
     # Python 2 support:
     __nonzero__ = __bool__
+
+
+    def __eq__(self, other):
+        return self.url_type == other.url_type \
+           and self.type_value == other.type_value
+
+
+    def __ne__(self, other):
+        return self.url_type != other.url_type \
+            or self.type_value != other.type_value
+
+
+    def __getstate__(self):
+        """
+        Pickle support
+        """
+        # Avoid pickling the registry if it's the shared one.
+        from any_urlfield.models.fields import AnyUrlField
+        if self.url_type_registry != AnyUrlField._static_registry:
+            url_type_registry = self.url_type_registry
+        else:
+            url_type_registry = None
+
+        return (url_type_registry, self.url_type.prefix, self.type_value)
+
+
+    def __setstate__(self, state):
+        url_type_registry, prefix, type_value = state
+
+        from any_urlfield.models.fields import AnyUrlField
+        if url_type_registry is not None:
+            self.url_type_registry = url_type_registry
+        else:
+            self.url_type_registry = AnyUrlField._static_registry
+
+        self.type_value = type_value
+        self.url_type = self.url_type_registry[prefix]
